@@ -107,6 +107,7 @@ use AutoLoader;
 	CDF_INT1
 	CDF_INT2
 	CDF_INT4
+	CDF_INT8
 	CDF_INTERNAL_ERROR
 	CDF_MAJORITY_
 	CDF_MAXREC_
@@ -135,6 +136,7 @@ use AutoLoader;
 	CDF_RELEASE_
 	CDF_SCRATCHDIR_
 	CDF_STATUS_
+	CDF_TIME_TT2000
 	CDF_UCHAR
 	CDF_UINT1
 	CDF_UINT2
@@ -160,6 +162,8 @@ use AutoLoader;
 	CURzENTRY_EXISTENCE_
 	BACKWARDFILEon
 	BACKWARDFILEoff
+	VALIDATEFILEon
+	VALIDATEFILEoff
 	DATATYPE_SIZE_
 	DECOMPRESSION_ERROR
 	DECSTATION_DECODING
@@ -193,6 +197,8 @@ use AutoLoader;
 	ILLEGAL_FOR_SCOPE
 	ILLEGAL_IN_zMODE
 	ILLEGAL_ON_V1_CDF
+	ILLEGAL_TT2000_VALUE
+	IS_A_NETCDF
 	LIB_COPYRIGHT_
 	LIB_INCREMENT_
 	LIB_RELEASE_
@@ -532,17 +538,17 @@ FCN: {
 	    if ($item == &ATTR_ || $item == &CDF_CACHESIZE_ || 
 		$item == &CDF_DECODING_ ||
 		$item == &CDF_NEGtoPOSfp0_MODE_ || 
-		$item == &CDF_READONLY_MODE_ || $item == &CDF_STATUS_ ||
+		$item == &CDF_READONLY_MODE_ || $item == &CDF_STATUS_ || 
 		$item == &CDF_zMODE_ || $item == &COMPRESS_CACHESIZE_ || 
-		$item == &gENTRY_ || $item == &rENTRY_ ||
-		$item == &rVAR_ || $item == &rVAR_CACHESIZE_ ||
+		$item == &gENTRY_ || $item == &rENTRY_ || 
+		$item == &rVAR_ || $item == &rVAR_CACHESIZE_ || 
 		$item == &rVAR_RESERVEPERCENT_ || 
 		$item == &rVARs_RECCOUNT_ || $item == &rVARs_RECINTERVAL_ || 
-		$item == &rVARs_RECNUMBER_ || $item == &STAGE_CACHESIZE_ ||
+		$item == &rVARs_RECNUMBER_ || $item == &STAGE_CACHESIZE_ || 
 		$item == &zENTRY_ || $item == &zVAR_ || 
 		$item == &zVAR_CACHESIZE_ || $item == &zVAR_RECCOUNT_ || 
-		$item == &zVAR_RECINTERVAL_ || $item == &zVAR_RECNUMBER_ ||
-                $item == &rVARs_CACHESIZE_ || $item == &zVARs_CACHESIZE_ ||
+		$item == &zVAR_RECINTERVAL_ || $item == &zVAR_RECNUMBER_ || 
+                $item == &rVARs_CACHESIZE_ || $item == &zVARs_CACHESIZE_ || 
 		$item == &zVAR_RESERVEPERCENT_ ) {
 		   my $LongRef = shift;
 		   $status = CDFlib5($operation, $item, $$LongRef);
@@ -564,7 +570,7 @@ FCN: {
 			$status = CDFlibnone($operation, $item);
 
 	    } elsif ($item == &gENTRY_EXISTENCE_ || 
-		     $item == &rENTRY_EXISTENCE_ ||
+		     $item == &rENTRY_EXISTENCE_ || 
 		     $item == &zENTRY_EXISTENCE_) {
 			my $Long = shift;
 			$status = CDFlibl($operation, $item, $Long);
@@ -575,9 +581,9 @@ FCN: {
 			$status = CDFlib51($operation, $item, $$LongRef, 
 					   $LongArrayRef);
 	    } elsif ($item == &rVARs_DIMCOUNTS_ || 
-		     $item == &rVARs_DIMINDICES_ ||
+		     $item == &rVARs_DIMINDICES_ || 
 		     $item == &rVARs_DIMINTERVALS_ || 
-		     $item == &zVAR_DIMCOUNTS_ ||
+		     $item == &zVAR_DIMCOUNTS_ || 
 		     $item == &zVAR_DIMINDICES_ || 
 		     $item == &zVAR_DIMINTERVALS_) {
 			my $LongArrayRef = shift;
@@ -1295,26 +1301,92 @@ sub CDFgetFileBackward {
         use strict; 
         my $flag;
         $flag = CDFgetFileBackwardX();
-	return $flag;
+        return $flag;
 }#end sub CDFgetFileBackward
         
 sub CDFsetChecksum {
         use strict;
         my $id = shift;
         my $chksum = shift;
-	my $status;
-	$status = CDFsetChecksumX($id, $chksum);
+        my $status;
+        $status = CDFsetChecksumX($id, $chksum);
         return $status;
 }#end sub CDFsetChecksum
 
 sub CDFgetChecksum {
         use strict;
         my $id = shift;
-	my $chksum = shift;
+        my $chksum = shift;
         my $status;
         $status = CDFgetChecksumX($id, $$chksum);
         return $status;
 }#end sub CDFgetChecksum
 
+sub CDFsetValidate {
+        use strict;
+        my $validateX = shift;
+        CDFsetValidateX($validateX);
+}#end sub CDFsetValidate
+
+sub CDFgetValidate {
+        use strict;
+        my $validateX;
+        $validateX = CDFgetValidateX();
+        return $validateX;
+}#end sub CDFgetValidate
+
+sub computeTT2000 {
+        use strict;
+        use Math::BigInt;
+        my $year = shift;
+        my $month = shift;
+        my $day = shift;
+        my $hour = shift;
+        my $minute = shift;
+        my $second = shift;
+        my $milsec = shift;
+        my $micsec = shift;
+        my $nansec = shift;
+        my $tt2000Str;
+        computeTT2000X ($year, $month, $day, $hour, $minute, $second, $milsec,
+                        $micsec, $nansec, $tt2000Str); 
+        my $tt2000 = Math::BigInt->new($tt2000Str);
+        return $tt2000;
+}#end sub computeTT2000
+
+sub parseTT2000 {
+        use strict;
+        use Math::BigInt;
+        my $tt2000String = shift;
+        my $tt2000Str;
+        parseTT2000X ($tt2000String, $tt2000Str);
+        my $tt2000 = Math::BigInt->new($tt2000Str);
+        return $tt2000;
+}#end sub parseTT2000
+
+sub encodeTT2000 {
+        use strict;
+        my $tt2000 = $_[0];
+        my $format;
+        if (scalar(@_) == 3) {
+          $format = $_[2];
+        } else {
+          $format = 3;
+        }
+        encodeTT2000X ($tt2000, $_[1], $format);
+}#end sub encodeTT2000
+
+sub leapsecondsinfo {
+        use strict;
+        my $debug;
+        if (scalar(@_) == 1) {
+          my $value;
+          $value = shift;
+          $debug = int($value);
+        } else {
+          $debug = 0;
+        }
+        leapsecondsinfoX ($debug);
+}#end sub leapseonds_infoX
 
 
