@@ -36,7 +36,18 @@
 *   V3.5d 14-Dec-97, J Love	Added ALPHAVMSi encoding.
 *   V3.6  08-Apr-04, M Liu      Added  new data type CDF_EPOCH16 and some
 *                               epoch functions related to the new type.
-*
+*   V3.7  28-Apr-09, M Liu      Modified MAC_ENCODING/DECODEING to PPC_ENCODING
+*                               /DECODING as Mac and Linux can run on PPC box.
+*   V3.8  10-Dec-10, M Liu      Added encodeEPOCH4, encodeEPOCH16_4,
+*                               parseEPOCH4, parseEPOCH16_4 to handle epochs
+*                               conforming to ISO 8601.
+*   V3.9  04-Apr-11, M Liu      Added a few new functions for TT2000 epoch.
+*   V3.10 03-Jan-12, M Liu      Added CDFgetzVarAllRecordsByVarID,
+*                               CDFgetzVarRangeRecordsByVarID,
+*                               CDFgetVarAllRecordsByVarName,
+*                               CDFgetVarRangeRecordsByVarName functions, and
+*                               a set of similar functions for put operations.
+*                               Added a new error message.
 ******************************************************************************/
 
 #if !defined(CDFh_INCLUDEd__)
@@ -49,6 +60,12 @@
 typedef void *CDFid;
 typedef long CDFstatus;
 
+/******************************************************************************
+* CDF defined variables
+******************************************************************************/
+
+static double *TT2000NULL = 0;
+
 /******************************************************************************
 * Limits
 ******************************************************************************/
@@ -76,11 +93,18 @@ typedef long CDFstatus;
 #define EPOCH1_STRING_LEN	16
 #define EPOCH2_STRING_LEN	14
 #define EPOCH3_STRING_LEN	24
+#define EPOCH4_STRING_LEN	23
 
 #define EPOCH16_STRING_LEN      36
 #define EPOCH16_1_STRING_LEN    24
 #define EPOCH16_2_STRING_LEN    14
 #define EPOCH16_3_STRING_LEN    36
+#define EPOCH16_4_STRING_LEN    32
+
+#define TT2000_0_STRING_LEN     30
+#define TT2000_1_STRING_LEN     19
+#define TT2000_2_STRING_LEN     14
+#define TT2000_3_STRING_LEN     29
 
 #define EPOCHx_STRING_MAX	50
 #define EPOCHx_FORMAT_MAX	68
@@ -92,6 +116,7 @@ typedef long CDFstatus;
 #define CDF_INT1		1L
 #define CDF_INT2		2L
 #define CDF_INT4		4L
+#define CDF_INT8		8L
 #define CDF_UINT1		11L
 #define CDF_UINT2		12L
 #define CDF_UINT4		14L
@@ -99,6 +124,8 @@ typedef long CDFstatus;
 #define CDF_REAL8		22L
 #define CDF_EPOCH		31L	/* Standard style. */
 #define CDF_EPOCH16		32L	/* Extended style. */
+#define CDF_TIME_TT2000		33L	/* One more style with leap seconds
+					   and J2000 base time. */
 #define CDF_BYTE		41L     /* same as CDF_INT1 (signed) */
 #define CDF_FLOAT		44L     /* same as CDF_REAL4 */
 #define CDF_DOUBLE		45L     /* same as CDF_REAL8 */
@@ -117,7 +144,7 @@ typedef long CDFstatus;
 #define IBMPC_ENCODING          6L
 #define IBMRS_ENCODING          7L
 #define HOST_ENCODING           8L
-#define MAC_ENCODING            9L
+#define PPC_ENCODING            9L
 #define HP_ENCODING             11L
 #define NeXT_ENCODING           12L
 #define ALPHAOSF1_ENCODING      13L
@@ -137,7 +164,9 @@ typedef long CDFstatus;
 #define IBMPC_DECODING          IBMPC_ENCODING
 #define IBMRS_DECODING          IBMRS_ENCODING
 #define HOST_DECODING           HOST_ENCODING
-#define MAC_DECODING            MAC_ENCODING
+#define PPC_DECODING            PPC_ENCODING
+#define MAC_ENCODING            PPC_ENCODING
+#define MAC_DECODING            PPC_ENCODING
 #define HP_DECODING             HP_ENCODING
 #define NeXT_DECODING           NeXT_ENCODING
 #define ALPHAOSF1_DECODING      ALPHAOSF1_ENCODING
@@ -188,6 +217,13 @@ typedef long CDFstatus;
 #define READONLYon              (-1L)
 #define READONLYoff             0L
 
+/******************************************************************************
+* Validate data modes.
+******************************************************************************/
+
+#define VALIDATEFILEon          (-1L)
+#define VALIDATEFILEoff         0L
+
 /******************************************************************************
 * zModes.
 ******************************************************************************/
@@ -243,6 +279,8 @@ typedef long CDFstatus;
 						   hasn't been selected yet. */
 
 #define ILLEGAL_EPOCH_VALUE	(-1.0)
+#define ILLEGAL_TT2000_VALUE    (-9223372036854775805LL)
+#define FILLED_TT2000_VALUE	(-9223372036854775807LL-1)
 
 /******************************************************************************
 * Status codes (CDFstatus)
@@ -388,7 +426,10 @@ typedef long CDFstatus;
 #define BAD_CHECKSUM                    ((CDFstatus) (-2225)) 
 #define CHECKSUM_ERROR                  ((CDFstatus) (-2226))
 #define CHECKSUM_NOT_ALLOWED            ((CDFstatus) (-2227))
-/* #define ZLIB_DECOMPRESSION_ERROR        ((CDFstatus) (-2228)) */
+#define IS_A_NETCDF                     ((CDFstatus) (-2228))
+#define TT2000_TIME_ERROR               ((CDFstatus) (-2229))
+#define UNABLE_TO_PROCESS_CDF           ((CDFstatus) (-2230))
+/* #define ZLIB_DECOMPRESSION_ERROR        ((CDFstatus) (-2231)) */
 
 /******************************************************************************
 * Functions (for INTERNAL interface).
@@ -409,6 +450,9 @@ typedef long CDFstatus;
 #define GETCDFFILEBACKWARD_     1011L
 #define CHECKSUM_               1012L
 #define GETCDFCHECKSUM_         1013L
+#define VALIDATE_               1014L
+#define GETCDFVALIDATE_         1015L
+#define GETLEAPSECONDSENVVAR_   1016L
 
 #define NULL_			1000L
 
@@ -577,6 +621,8 @@ typedef long CDFstatus;
 #define CDFwithSTATS_		200L	/* For CDF internal use only! */
 #define CDF_ACCESS_		201L	/* For CDF internal use only! */
 
+#define TT2000END 		-99999.999
+
 /******************************************************************************
 * C interface macros.
 ******************************************************************************/
@@ -590,6 +636,7 @@ typedef long CDFstatus;
 #define CDFopenCDF CDFopen
 #define CDFdeleteCDF CDFdelete
 #define CDFcloseCDF CDFclose
+#define CDFselectCDF CDFselect
 
 #define CDFattrEntryInquire(id,attrNum,entryNum,dataType,numElems) \
 CDFinquireAttrEntry(id,0,attrNum,entryNum,dataType,numElems)
@@ -705,292 +752,133 @@ CDFgetAttrEntryNumElements(id,3,attrNum,entryNum,numElems)
 #define CDFgetNumrVars(id,numVars) CDFgetNumVars(id,0,numVars)
 #define CDFgetNumzVars(id,numVars) CDFgetNumVars(id,1,numVars)
 
-/*#define CDFdeleterVar(id,varNum) CDFdeleteVar(id,0,varNum)*/
 #define CDFdeletezVar(id,varNum) CDFdeleteVar(id,1,varNum)
 
-/*
-#define CDFdeleterVarRecords(id,varNum,sRec,eRec) \
-CDFdeleteVarRecords(id,0,varNum,sRec,eRec)
-*/
 #define CDFdeletezVarRecords(id,varNum,sRec,eRec) \
 CDFdeleteVarRecords(id,1,varNum,sRec,eRec)
 
-/*
-#define CDFgetrVarName(id,varNum,varName) \
-CDFgetVarName(id,0,varNum,varName)
-*/
 #define CDFgetzVarName(id,varNum,varName) \
 CDFgetVarName(id,1,varNum,varName)
 
-/*
-#define CDFgetrVarMaxWrittenRecNum(id,varNum,maxRec) \
-CDFgetVarMaxWrittenRecNum(id,0,varNum,maxRec)
-*/
 #define CDFgetzVarMaxWrittenRecNum(id,varNum,maxRec) \
 CDFgetVarMaxWrittenRecNum(id,1,varNum,maxRec)
 
-/*
-#define CDFgetrVarsMaxWrittenRecNum(id,maxRec) \
-CDFgetVarsMaxWrittenRecNum(id,0,maxRec)
-*/
 #define CDFgetzVarsMaxWrittenRecNum(id,maxRec) \
 CDFgetVarsMaxWrittenRecNum(id,1,maxRec)
 
-/*
-#define CDFgetrVarMaxAllocRecNum(id,varNum,maxAllocRec) \
-CDFgetVarMaxAllocRecNum(id,0,varNum,maxAllocRec)
-*/
 #define CDFgetzVarMaxAllocRecNum(id,varNum,maxAllocRec) \
 CDFgetVarMaxAllocRecNum(id,1,varNum,maxAllocRec)
 
-/*
-#define CDFgetrVarDataType(id,varNum,dataType) \
-CDFgetVarDataType(id,0,varNum,dataType)
-*/
 #define CDFgetzVarDataType(id,varNum,dataType) \
 CDFgetVarDataType(id,1,varNum,dataType)
 
-/*
-#define CDFgetrVarAllocRecords(id,varNum,allocRecs) \
-CDFgetVarAllocRecords(id,0,varNum,allocRecs)
-*/
 #define CDFgetzVarAllocRecords(id,varNum,allocRecs) \
 CDFgetVarAllocRecords(id,1,varNum,allocRecs)
-/*
-#define CDFsetrVarAllocRecords(id,varNum,allocRecs) \
-CDFsetVarAllocRecords(id,0,varNum,allocRecs)
-*/
 #define CDFsetzVarAllocRecords(id,varNum,allocRecs) \
 CDFsetVarAllocRecords(id,1,varNum,allocRecs)
 
-/*
-#define CDFsetrVarAllocBlockRecords(id,varNum,firstRec,lastRec) \
-CDFsetVarAllocBlockRecords(id,0,varNum,firstRec,lastRec)
-*/
 #define CDFsetzVarAllocBlockRecords(id,varNum,firstRec,lastRec) \
 CDFsetVarAllocBlockRecords(id,1,varNum,firstRec,lastRec)
 
-/*
-#define CDFgetrVarBlockingFactor(id,varNum,bf) \
-CDFgetVarBlockingFactor(id,0,varNum,bf)
-*/
 #define CDFgetzVarBlockingFactor(id,varNum,bf) \
 CDFgetVarBlockingFactor(id,1,varNum,bf)
-/*
-#define CDFsetrVarBlockingFactor(id,varNum,bf) \
-CDFsetVarBlockingFactor(id,0,varNum,bf)
-*/
 #define CDFsetzVarBlockingFactor(id,varNum,bf) \
 CDFsetVarBlockingFactor(id,1,varNum,bf)
 
-/*
-#define CDFgetrVarCompression(id,varNum,cType,cParms,cPct) \
-CDFgetVarCompression(id,0,varNum,cType,cParms,cPct)
-*/
 #define CDFgetzVarCompression(id,varNum,cType,cParms,cPct) \
 CDFgetVarCompression(id,1,varNum,cType,cParms,cPct)
-/*
-#define CDFsetrVarCompression(id,varNum,cType,cParms) \
-CDFsetVarCompression(id,0,varNum,cType,cParms)
-*/
 #define CDFsetzVarCompression(id,varNum,cType,cParms) \
 CDFsetVarCompression(id,1,varNum,cType,cParms)
 
-/*
-#define CDFsetrVarDataSpec(id,varNum,dataType) \
-CDFsetVarDataSpec(id,0,varNum,dataType,(long)-99)
-*/
 #define CDFsetzVarDataSpec(id,varNum,dataType) \
 CDFsetVarDataSpec(id,1,varNum,dataType,(long)-99)
 
-/*
-#define CDFsetrVarDimVariances(id,varNum,dimVarys) \
-CDFsetVarDimVariances(id,0,varNum,dimVarys)
-*/
 #define CDFsetzVarDimVariances(id,varNum,dimVarys) \
 CDFsetVarDimVariances(id,1,varNum,dimVarys)
 
-/*
-#define CDFgetrVarDimVariances(id,varNum,dimVarys) \
-CDFgetVarDimVariances(id,0,varNum,dimVarys)
-*/
 #define CDFgetzVarDimVariances(id,varNum,dimVarys) \
 CDFgetVarDimVariances(id,1,varNum,dimVarys)
 
-/*
-#define CDFgetrVarNumElements(id,varNum,numEles) \
-CDFgetVarNumElements(id,0,varNum,numEles)
-*/
 #define CDFgetzVarNumElements(id,varNum,numEles) \
 CDFgetVarNumElements(id,1,varNum,numEles)
 
-/*
-#define CDFgetrVarNumRecsWritten(id,varNum,numRecs) \
-CDFgetVarNumRecsWritten(id,0,varNum,numRecs)
-*/
 #define CDFgetzVarNumRecsWritten(id,varNum,numRecs) \
 CDFgetVarNumRecsWritten(id,1,varNum,numRecs)
 
-/*
-#define CDFsetrVarInitialRecs(id,varNum,initRecs) \
-CDFsetVarInitialRecs(id,0,varNum,initRecs)
-*/
 #define CDFsetzVarInitialRecs(id,varNum,initRecs) \
 CDFsetVarInitialRecs(id,1,varNum,initRecs)
 
-/*
-#define CDFgetrVarPadValue(id,varNum,pad) \
-CDFgetVarPadValue(id,0,varNum,pad)
-*/
 #define CDFgetzVarPadValue(id,varNum,pad) \
 CDFgetVarPadValue(id,1,varNum,pad)
-/*
-#define CDFsetrVarPadValue(id,varNum,pad) \
-CDFsetVarPadValue(id,0,varNum,pad)
-*/
 #define CDFsetzVarPadValue(id,varNum,pad) \
 CDFsetVarPadValue(id,1,varNum,pad)
 
-/*
-#define CDFgetrVarRecVariance(id,varNum,recVary) \
-CDFgetVarRecVariance(id,0,varNum,recVary)
-*/
 #define CDFgetzVarRecVariance(id,varNum,recVary) \
 CDFgetVarRecVariance(id,1,varNum,recVary)
-/*
-#define CDFsetrVarRecVariance(id,varNum,recVary) \
-CDFsetVarRecVariance(id,0,varNum,recVary)
-*/
 #define CDFsetzVarRecVariance(id,varNum,recVary) \
 CDFsetVarRecVariance(id,1,varNum,recVary)
 
-/*
-#define CDFgetrVarSeqData(id,varNum,data) \
-CDFgetVarSeqData(id,0,varNum,data)
-*/
 #define CDFgetzVarSeqData(id,varNum,data) \
 CDFgetVarSeqData(id,1,varNum,data)
-/*
-#define CDFputrVarSeqData(id,varNum,data) \
-CDFputVarSeqData(id,0,varNum,data)
-*/
 #define CDFputzVarSeqData(id,varNum,data) \
 CDFputVarSeqData(id,1,varNum,data)
 
-/*
-#define CDFgetrVarSparseRecords(id,varNum,sprecs) \
-CDFgetVarSparseRecords(id,0,varNum,sprecs)
-*/
 #define CDFgetzVarSparseRecords(id,varNum,sprecs) \
 CDFgetVarSparseRecords(id,1,varNum,sprecs)
-/*
-#define CDFsetrVarSparseRecords(id,varNum,sprecs) \
-CDFsetVarSparseRecords(id,0,varNum,sprecs)
-*/
 #define CDFsetzVarSparseRecords(id,varNum,sprecs) \
 CDFsetVarSparseRecords(id,1,varNum,sprecs)
 
-/*
-#define CDFgetrVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
-CDFgetVarsRecordDatabyNames(id,0,numVars,varNames,recNum,bufferPtr)
-*/
 #define CDFgetzVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
 CDFgetVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtr)
-/*
-#define CDFputrVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
-CDFputVarsRecordDatabyNames(id,0,numVars,varNames,recNum,bufferPtr)
-*/
 #define CDFputzVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
 CDFputVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtr)
 
-/*
-#define CDFgetrVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
-CDFgetVarsRecordDatabyNumbers(id,0,numVars,varNumbers,recNum,buffer)
-*/
 #define CDFgetzVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
 CDFgetVarsRecordDatabyNumbers(id,1,numVars,varNumbers,recNum,buffer)
-/*
-#define CDFputrVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
-CDFputVarsRecordDatabyNumbers(id,0,numVars,varNumbers,recNum,buffer)
-*/
 #define CDFputzVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
 CDFputVarsRecordDatabyNumbers(id,1,numVars,varNumbers,recNum,buffer)
 
-/*
-#define CDFgetrVarRecordData(id,varNum,recNum,buffer) \
-CDFgetVarRecordData(id,0,varNum,recNum,buffer)
-*/
 #define CDFgetzVarRecordData(id,varNum,recNum,buffer) \
 CDFgetVarRecordData(id,1,varNum,recNum,buffer)
-/*
-#define CDFputrVarRecordData(id,varNum,recNum,buffer) \
-CDFputVarRecordData(id,0,varNum,recNum,buffer)
-*/
 #define CDFputzVarRecordData(id,varNum,recNum,buffer) \
 CDFputVarRecordData(id,1,varNum,recNum,buffer)
 
-/*
-#define CDFsetrVarCacheSize(id,varNum,numBuffers) \
-CDFsetVarCacheSize(id,0,varNum,numBuffers)
-*/
 #define CDFsetzVarCacheSize(id,varNum,numBuffers) \
 CDFsetVarCacheSize(id,1,varNum,numBuffers)
 
-/*
-#define CDFsetrVarsCacheSize(id,numBuffers) \
-CDFsetVarsCacheSize(id,0,numBuffers)
-*/
 #define CDFsetzVarsCacheSize(id,numBuffers) \
 CDFsetVarsCacheSize(id,1,numBuffers)
 
-/*
-#define CDFgetrVarCacheSize(id,varNum,numBuffers) \
-CDFgetVarCacheSize(id,0,varNum,numBuffers)
-*/
 #define CDFgetzVarCacheSize(id,varNum,numBuffers) \
 CDFgetVarCacheSize(id,1,varNum,numBuffers)
 
-/*
-#define CDFconfirmrVarExistence(id,varName) \
-CDFconfirmVarExistence(id,0,varName)
-*/
 #define CDFconfirmzVarExistence(id,varName) \
 CDFconfirmVarExistence(id,1,varName)
 
-/*
-#define CDFconfirmrVarPadValueExistence(id,varNum) \
-CDFconfirmVarPadValueExistence(id,0,varNum)
-*/
 #define CDFconfirmzVarPadValueExistence(id,varNum) \
 CDFconfirmVarPadValueExistence(id,1,varNum)
 
-/*
-#define CDFgetrVarReservePercent(id,varNum,percent) \
-CDFgetVarReservePercent(id,0,varNum,percent)
-*/
 #define CDFgetzVarReservePercent(id,varNum,percent) \
 CDFgetVarReservePercent(id,1,varNum,percent)
 
-/*
-#define CDFsetrVarReservePercent(id,varNum,percent) \
-CDFsetVarReservePercent(id,0,varNum,percent)
-*/
 #define CDFsetzVarReservePercent(id,varNum,percent) \
 CDFsetVarReservePercent(id,1,varNum,percent)
 
-/*
-#define CDFgetrVarSeqPos(id,varNum,recNum,indices) \
-CDFgetVarSeqPos(id,0,varNum,recNum,indices)
-*/
 #define CDFgetzVarSeqPos(id,varNum,recNum,indices) \
 CDFgetVarSeqPos(id,1,varNum,recNum,indices)
-/*
-#define CDFsetrVarSeqPos(id,varNum,recNum,indices) \
-CDFsetVarSeqPos(id,0,varNum,recNum,indices)
-*/
 #define CDFsetzVarSeqPos(id,varNum,recNum,indices) \
 CDFsetVarSeqPos(id,1,varNum,recNum,indices)
 
+#define CDFgetzVarAllRecordsByVarID(id,varNum,buffer) \
+CDFgetVarAllRecordsByVarID(id,1,varNum,buffer)
+#define CDFputzVarAllRecordsByVarID(id,varNum,numRecs,buffer) \
+CDFputVarAllRecordsByVarID(id,1,varNum,numRecs,buffer)
+
+#define CDFgetzVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
+CDFgetVarRangeRecordsByVarID(id,1,varNum,startRec,stopRec,buffer)
+#define CDFputzVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
+CDFputVarRangeRecordsByVarID(id,1,varNum,startRec,stopRec,buffer)
+
 /*
  * CLOSE_  *
  *         */
@@ -1604,6 +1492,10 @@ CDFlib (SELECT_, CDF_, id, \
  * SELECT_ *
  *         */
 
+#define CDFselect(id) \
+CDFlib (SELECT_, CDF_, id, \
+        NULL_)
+
 #define CDFsetDecoding(id,decoding) \
 CDFlib (SELECT_, CDF_, id, \
                  CDF_DECODING_, decoding, \
@@ -1662,6 +1554,16 @@ CDFlib (SELECT_, CDF_, id, \
                  CDF_zMODE_, zMode, \
         NULL_)
 
+/******************************************************************************
+* TT2000 macros define'd
+******************************************************************************/
+
+#define parseTT2000 CDF_TT2000_from_UTC_string
+#define encodeTT2000 CDF_TT2000_to_UTC_string
+#define computeTT2000 CDF_TT2000_from_UTC_parts
+#define breakdownTT2000 CDF_TT2000_to_UTC_parts
+#define TT2000breakdown CDF_TT2000_to_UTC_parts
+
 /******************************************************************************
 * Function prototypes.
 *     It is assumed that `__cplusplus' is defined for ALL C++ compilers.  If
@@ -1788,6 +1690,32 @@ VISIBLE_PREFIX CDFstatus CDFputVarsRecordDatabyNames PROTOARGs((
   CDFid id, int zVar, long numVars, char *varNames[], long recNum,
   void *buffer[]
 ));
+VISIBLE_PREFIX CDFstatus CDFgetVarAllRecordsByVarID PROTOARGs((
+  CDFid id, int zVar, long varNum, void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarID PROTOARGs((
+  CDFid id, int zVar, long varNum, long numRec, void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFgetVarRangeRecordsByVarID PROTOARGs((
+  CDFid id, int zVar, long varNum, long startRec, long stopRec,
+  void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFputVarRangeRecordsByVarID PROTOARGs((
+  CDFid id, int zVar, long varNum, long startRec, long stopRec,
+  void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFgetVarAllRecordsByVarName PROTOARGs((
+  CDFid id, char *varName, void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarName PROTOARGs((
+  CDFid id, char *varName, long numRecs, void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFgetVarRangeRecordsByVarName PROTOARGs((
+  CDFid id, char *varName, long startRec, long stopRec, void *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFputVarRangeRecordsByVarName PROTOARGs((
+  CDFid id, char *varName, long startRec,long stopRec, void *buffer
+));
 VISIBLE_PREFIX void CDFsetFileBackward PROTOARGs((
   int flag
 ));
@@ -1801,6 +1729,9 @@ VISIBLE_PREFIX void CDFsetChecksumMode PROTOARGs((
 ));
 VISIBLE_PREFIX long CDFgetChecksumMode PROTOARGs(());
 VISIBLE_PREFIX int CDFgetFileBackwardEnvVar PROTOARGs(());
+VISIBLE_PREFIX void CDFsetValidate PROTOARGs((long mode));
+VISIBLE_PREFIX int CDFgetValidate PROTOARGs(());
+VISIBLE_PREFIX int CDFgetValidateDebug PROTOARGs(());
 #if !defined(__CFM68K__) || defined(__USING_STATIC_LIBS__) || !defined(CFM68KDLL)
 VISIBLE_PREFIX void EPOCHbreakdown PROTOARGs((
   double epoch, long *year, long *month, long *day, long *hour, long *minute,
@@ -1814,6 +1745,7 @@ VISIBLE_PREFIX double parseEPOCH PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH1 PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH2 PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH3 PROTOARGs((char *inString));
+VISIBLE_PREFIX double parseEPOCH4 PROTOARGs((char *inString));
 VISIBLE_PREFIX void encodeEPOCH PROTOARGs((
   double epoch, char epString[EPOCH_STRING_LEN+1]
 ));
@@ -1826,45 +1758,99 @@ VISIBLE_PREFIX void encodeEPOCH2 PROTOARGs((
 VISIBLE_PREFIX void encodeEPOCH3 PROTOARGs((
   double epoch, char epString[EPOCH3_STRING_LEN+1]
 ));
+VISIBLE_PREFIX void encodeEPOCH4 PROTOARGs((
+  double epoch, char epString[EPOCH4_STRING_LEN+1]
+));
 VISIBLE_PREFIX void encodeEPOCHx PROTOARGs((
   double epoch, char format[EPOCHx_FORMAT_MAX],
   char encoded[EPOCHx_STRING_MAX]
 ));
 VISIBLE_PREFIX void EPOCH16breakdown PROTOARGs((
-  double epoch[], long *year, long *month, long *day, long *hour,
+  double *epoch, long *year, long *month, long *day, long *hour,
   long *minute, long *second, long *msec, long *usec, long *nsec, long *psec
 ));
 VISIBLE_PREFIX double computeEPOCH16 PROTOARGs((
   long year, long month, long day, long hour, long minute, long second,
-  long msec, long usec, long nsec, long psec, double epoch[]
+  long msec, long usec, long nsec, long psec, double *epoch
 ));
 VISIBLE_PREFIX double parseEPOCH16 PROTOARGs((char *inString,
-  double epoch[]
+  double *epoch
 ));
 VISIBLE_PREFIX double parseEPOCH16_1 PROTOARGs((char *inStringch,
-  double epoch[]
+  double *epoch
 ));
 VISIBLE_PREFIX double parseEPOCH16_2 PROTOARGs((char *inStringch,
-  double epoch[]
+  double *epoch
 ));
 VISIBLE_PREFIX double parseEPOCH16_3 PROTOARGs((char *inStringch,
-  double epoch[]
+  double *epoch
+));
+VISIBLE_PREFIX double parseEPOCH16_4 PROTOARGs((char *inStringch,
+  double *epoch
 ));
 VISIBLE_PREFIX void encodeEPOCH16 PROTOARGs((
-  double epoch[], char epString[EPOCH16_STRING_LEN+1]
+  double *epoch, char epString[EPOCH16_STRING_LEN+1]
 ));
 VISIBLE_PREFIX void encodeEPOCH16_1 PROTOARGs((
-  double epoch[], char epString[EPOCH16_1_STRING_LEN+1]
+  double *epoch, char epString[EPOCH16_1_STRING_LEN+1]
 ));
 VISIBLE_PREFIX void encodeEPOCH16_2 PROTOARGs((
-  double epoch[], char epString[EPOCH16_2_STRING_LEN+1]
+  double *epoch, char epString[EPOCH16_2_STRING_LEN+1]
 ));
 VISIBLE_PREFIX void encodeEPOCH16_3 PROTOARGs((
-  double epoch[], char epString[EPOCH16_3_STRING_LEN+1]
+  double *epoch, char epString[EPOCH16_3_STRING_LEN+1]
+));
+VISIBLE_PREFIX void encodeEPOCH16_4 PROTOARGs((
+  double *epoch, char epString[EPOCH16_4_STRING_LEN+1]
 ));
 VISIBLE_PREFIX void encodeEPOCH16_x PROTOARGs((
-  double epoch[], char format[EPOCHx_FORMAT_MAX], char encoded[EPOCHx_STRING_MAX]
+  double *epoch, char format[EPOCHx_FORMAT_MAX],
+  char encoded[EPOCHx_STRING_MAX]
 ));
+/******************************************************************************
+* A new set of functions to handle CDF_TIME_TT2000 time type.
+******************************************************************************/
+
+VISIBLE_PREFIX void CDF_TT2000_to_UTC_parts PROTOARGs((
+  long long tt2000, double *year, double *month, double *day, ...
+));
+VISIBLE_PREFIX long long CDF_TT2000_from_UTC_parts PROTOARGs((
+  double year, double month, double day, ...
+));
+VISIBLE_PREFIX double CDF_TT2000_to_UTC_EPOCH PROTOARGs((
+  long long time
+));
+VISIBLE_PREFIX long long CDF_TT2000_from_UTC_EPOCH PROTOARGs((
+  double epoch
+));
+VISIBLE_PREFIX double CDF_TT2000_to_UTC_EPOCH16 PROTOARGs((
+  long long time, double *epoch16
+));
+VISIBLE_PREFIX long long CDF_TT2000_from_UTC_EPOCH16 PROTOARGs((
+  double *epoch16
+));
+VISIBLE_PREFIX void CDF_TT2000_to_UTC_string PROTOARGs((
+  long long time, char *string, ...
+));
+VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string PROTOARGs((
+  char *string
+));
+VISIBLE_PREFIX void CDFClearLeapSecondsTable PROTOARGs(());
+VISIBLE_PREFIX void CDFfillLeapSecondsTable PROTOARGs((
+  double **table
+));
+VISIBLE_PREFIX int CDFgetRowsinLeapSecondsTable PROTOARGs(());
+#if defined(vms)
+VISIBLE_PREFIX void CDFgetLastDateinLeapSecondsTBL PROTOARGs((
+  long *year, long *month, long *day
+));
+#else
+VISIBLE_PREFIX void CDFgetLastDateinLeapSecondsTable PROTOARGs((
+  long *year, long *month, long *day
+));
+#endif
+VISIBLE_PREFIX char *CDFgetLeapSecondsTableEnvVar PROTOARGs(());
+VISIBLE_PREFIX int CDFgetLeapSecondsTableStatus PROTOARGs(());
 #endif
 #if defined(__cplusplus)
 }
