@@ -1,4 +1,4 @@
-#!/usr/local/bin/perl -w
+#!/usr/bin/perl -w
 #
 # testPerlCDFsi
 #
@@ -35,9 +35,10 @@
 #############################################################################
 
 use strict;
+use Math::BigInt;
 
-BEGIN { unshift @INC,'/home/liu/PerlCDF33_1/blib/arch',
-                     '/home/liu/PerlCDF33_1/blib/lib'; }
+BEGIN { unshift @INC,'/home/liu/PerlCDF34_1/blib/arch',
+                     '/home/liu/PerlCDF34_1/blib/lib'; }
 use CDF;
 
 my $N_DIMS = 2;
@@ -454,6 +455,36 @@ QuitEPOCH ("42.7") if ($usec16Out != $usec) ;
 QuitEPOCH ("42.8") if ($nsec16Out != $nsec) ;
 QuitEPOCH ("42.9") if ($psec16Out != $psec) ;
 
+#############################################################################
+# Test TT2000 Routines
+#############################################################################
+print "Test TT2000...\n";
+$year = 2010;
+
+my $tt2000 = CDF::computeTT2000 ($year, $month, $day, $hour, $minute,  
+                                 $second, $msec, $usec, $nsec);
+my $ttString3True = '2010-10-13T01:02:03.987876765';
+my $ttString3;
+
+CDF::encodeTT2000 ($tt2000, $ttString3);
+QuitEPOCH ("50.0") if ($ttString3 ne $ttString3True);
+
+my $tt2000Out = CDF::parseTT2000 ($ttString3);
+QuitEPOCH ("51.0") if ($tt2000Out != $tt2000) ;
+
+my ($usecOut, $nsecOut);
+CDF::TT2000breakdown ($tt2000, $yearOut, $monthOut, $dayOut, $hourOut, 
+                      $minuteOut, $secondOut, $msecOut, $usecOut, $nsecOut);
+QuitEPOCH ("52.1") if ($yearOut != $year) ;
+QuitEPOCH ("52.2") if ($monthOut != $month) ;
+QuitEPOCH ("52.3") if ($dayOut != $day) ;
+QuitEPOCH ("52.4") if ($hourOut != $hour) ;
+QuitEPOCH ("52.5") if ($minuteOut != $minute) ;
+QuitEPOCH ("52.6") if ($secondOut != $second) ;
+QuitEPOCH ("52.7") if ($msecOut != $msec) ;
+QuitEPOCH ("52.8") if ($usecOut != $usec) ;
+QuitEPOCH ("52.9") if ($nsecOut != $nsec) ;
+
 #############################################################################
 # Successful completion.
 #############################################################################
